/* GNOME DB library
 * Copyright (C) 1999-2002 The GNOME Foundation.
 *
 * AUTHORS:
 *      Rodrigo Moya <rodrigo@gnome-db.org>
 *
 * This Library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this Library; see the file COPYING.LIB.  If not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libgnomedb/gnome-db-login.h>
#include <libgnomedb/gnome-db-login-dialog.h>

#include "gnome-db-intl.h"

struct _GnomeDbLoginDialogPrivate {
	GtkWidget *login;
};

static void gnome_db_login_dialog_class_init   (GnomeDbLoginDialogClass *klass);
static void gnome_db_login_dialog_init         (GnomeDbLoginDialog *dialog,
						GnomeDbLoginDialogClass *klass);
static void gnome_db_login_dialog_set_property (GObject *object,
						guint paramid,
						const GValue *value,
						GParamSpec *pspec);
static void gnome_db_login_dialog_get_property (GObject *object,
						guint param_id,
						GValue *value,
						GParamSpec *pspec);
static void gnome_db_login_dialog_finalize     (GObject *object);

enum {
	PROP_0,
	PROP_DSN,
	PROP_USERNAME,
	PROP_PASSWORD
};

static GObjectClass *parent_class = NULL;

/*
 * GnomeDbLoginDialog class implementation
 */

static void
gnome_db_login_dialog_class_init (GnomeDbLoginDialogClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	object_class->set_property = gnome_db_login_dialog_set_property;
	object_class->get_property = gnome_db_login_dialog_get_property;
	object_class->finalize = gnome_db_login_dialog_finalize;

	/* add class properties */
	g_object_class_install_property (
		object_class, PROP_DSN,
		g_param_spec_string ("dsn", NULL, NULL, NULL, G_PARAM_READABLE));
	g_object_class_install_property (
		object_class, PROP_USERNAME,
		g_param_spec_string ("username", NULL, NULL, NULL, G_PARAM_READABLE));
	g_object_class_install_property (
		object_class, PROP_PASSWORD,
		g_param_spec_string ("password", NULL, NULL, NULL, G_PARAM_READABLE));
}

static void
gnome_db_login_dialog_init (GnomeDbLoginDialog *dialog, GnomeDbLoginDialogClass *klass)
{
	g_return_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog));

	dialog->priv = g_new0 (GnomeDbLoginDialogPrivate, 1);

	gtk_dialog_add_button (GTK_DIALOG (dialog), _("Cancel"), GTK_RESPONSE_CANCEL);
	gtk_dialog_add_button (GTK_DIALOG (dialog), _("OK"), GTK_RESPONSE_OK);
	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);

	dialog->priv->login = gnome_db_login_new (NULL);
	gtk_widget_show (dialog->priv->login);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), dialog->priv->login,
			    TRUE, TRUE, 0);
}

static void
gnome_db_login_dialog_set_property (GObject *object,
				    guint param_id,
				    const GValue *value,
				    GParamSpec *pspec)
{
	GnomeDbLoginDialog *dialog = (GnomeDbLoginDialog *) object;

	g_return_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog));

	switch (param_id) {
	default :
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
		break;
	}
}

static void
gnome_db_login_dialog_get_property (GObject *object,
				    guint param_id,
				    GValue *value,
				    GParamSpec *pspec)
{
	GnomeDbLoginDialog *dialog = (GnomeDbLoginDialog *) object;

	g_return_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog));

	switch (param_id) {
	case PROP_DSN :
		g_value_set_string (
			value,
			gnome_db_login_get_dsn (GNOME_DB_LOGIN (dialog->priv->login)));
		break;
	case PROP_USERNAME :
		g_value_set_string (
			value,
			gnome_db_login_get_username (GNOME_DB_LOGIN (dialog->priv->login)));
		break;
	case PROP_PASSWORD :
		g_value_set_string (
			value,
			gnome_db_login_get_password (GNOME_DB_LOGIN (dialog->priv->login)));
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
		break;
	}
}

static void
gnome_db_login_dialog_finalize (GObject *object)
{
	GnomeDbLoginDialog *dialog = (GnomeDbLoginDialog *) object;

	g_return_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog));

	/* free memory */
	g_free (dialog->priv);
	dialog->priv = NULL;

	parent_class->finalize (object);
}

GType
gnome_db_login_dialog_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (GnomeDbLoginDialogClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) gnome_db_login_dialog_class_init,
			NULL,
			NULL,
			sizeof (GnomeDbLoginDialog),
			0,
			(GInstanceInitFunc) gnome_db_login_dialog_init
		};
		type = g_type_register_static (GTK_TYPE_DIALOG, "GnomeDbLoginDialog", &info, 0);
	}
	return type;
}

/**
 * gnome_db_login_dialog_new
 */
GtkWidget *
gnome_db_login_dialog_new (const gchar *title)
{
	GnomeDbLoginDialog *dialog;

	dialog = g_object_new (GNOME_DB_TYPE_LOGIN_DIALOG, NULL);
	gtk_window_set_title (GTK_WINDOW (dialog), title);

	return GTK_WIDGET (dialog);
}

/**
 * gnome_db_login_dialog_run
 */
gboolean
gnome_db_login_dialog_run (GnomeDbLoginDialog *dialog)
{
	g_return_val_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog), FALSE);

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
		return TRUE;

	return FALSE;
}

/**
 * gnome_db_login_dialog_get_dsn
 */
const gchar *gnome_db_login_dialog_get_dsn (GnomeDbLoginDialog *dialog)
{
	g_return_val_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog), NULL);
	return gnome_db_login_get_dsn (GNOME_DB_LOGIN (dialog->priv->login));
}

/**
 * gnome_db_login_dialog_get_username
 */
const gchar *gnome_db_login_dialog_get_username (GnomeDbLoginDialog *dialog)
{
	g_return_val_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog), NULL);
	return gnome_db_login_get_username (GNOME_DB_LOGIN (dialog->priv->login));
}

/**
 * gnome_db_login_dialog_get_password
 */
const gchar *gnome_db_login_dialog_get_password (GnomeDbLoginDialog *dialog)
{
	g_return_val_if_fail (GNOME_DB_IS_LOGIN_DIALOG (dialog), NULL);
	return gnome_db_login_get_password (GNOME_DB_LOGIN (dialog->priv->login));
}
